@@ -7,11 +7,13 @@ R_CallMethodDef callMethods[] = {
 {"dist2spectra", (DL_FUNC) &dist2spectra, 3},
 {"sankoff2", (DL_FUNC) &sankoff2, 4},
 {"sankoff3", (DL_FUNC) &sankoff3, 8},
+{"pNodes", (DL_FUNC) &pNodes, 6},
 {"sankoffQuartet", (DL_FUNC) &sankoffQuartet, 4},
 {"rowMin", (DL_FUNC) &rowMin, 3},
 {"rowMax", (DL_FUNC) &rowMax, 3},
 {"matpro", (DL_FUNC) &matpro, 6},
 {"LogLik", (DL_FUNC) &LogLik, 8},
+{"LogLik2", (DL_FUNC) &LogLik, 8},
 {"getPM", (DL_FUNC) &getPM, 4},
 {"getdPM", (DL_FUNC) &getdPM, 4},
 {"getdPM2", (DL_FUNC) &getdPM2, 4},
